#include <strings.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "common.h"
#include "http.h"

#define BUFFSIZE 1024

int is_http(char *url)
{
	if (strlen(url) < 8)
		return 0;

	if (!strncasecmp("http://",url,7))
		return 1;

	return 0;
}

int open_http(char *url)
{
	int fd;
	struct sockaddr_in server;
	static struct dvb_host *dvbhost = NULL;
	char c, buffer[BUFFSIZE], *pos;
	int lines = 0;

	if(!is_http(url))
		return -1;

	if (!dvbhost) {
		dvbhost = parse(&(url[7]), "80");
		dvbhost->socktype = SOCK_STREAM;
	}

	if (resolve(dvbhost, &server) < 0) {
		return -1;
	}

	if ((fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
		perror("socket");
		return -1;
	}

	if (connect(fd, (struct sockaddr*) &server, sizeof(server)) < 0) {
		perror("connect");
		return -1;
	}

	snprintf(buffer, BUFFSIZE-1, "GET /%s HTTP/1.0\n\n",dvbhost->location);
	buffer[BUFFSIZE-1] = 0;
	if (send(fd, buffer, strlen(buffer), 0) < 0) {
		perror("send");
		return -1;
	}

	pos = buffer;
	while (1) {
		if (recv(fd, &c, 1, 0) < 1) {
			perror("recv");
			exit(EXIT_FAILURE);
		}

		if (pos-buffer >= BUFFSIZE)
			pos = buffer;

		*(pos++) = c;

		if (c == 0x0a) {
			*(--pos) = 0;
			if (pos-buffer > 0 && *(--pos) == 0x0d)
				*pos = 0;

			if (pos-buffer == 0)
				break;

#ifdef DEBUG
			printf("%d. %s (%d)\n", lines, buffer, pos-buffer);
#endif
			if (lines == 0) {
				if (strncasecmp("HTTP/", buffer, 5)) {
					fprintf(stderr, "Wrong answer from server: %s\n", buffer);
					return -1;
				}

				pos = buffer;
				while (*pos != 0) {
					if (*(pos++) == ' ') {
						if(strncmp("200", pos, 3)) {
							fprintf(stderr, "Wrong result-code: %s\n", buffer);
							return -1;
						} else {
							break;
						}
					}

					if (*(++pos) == 0) {
						fprintf(stderr, "Wrong answer from server: %s\n", buffer);
						return -1;
					}
				}
			}

			pos = buffer;
			lines++;
		}
	}

	return fd;
}
