#include <features.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <strings.h>

#include "http.h"
#include "mcast.h"

#define CHUNKSIZE 3000
#define GTOD_INTERVAL 100

void record(int(*open_fn)(char *), char *url, char *outfile, int duration)
{
	struct timeval start, curr;
	int bytes, recvd, written, count = 0;
	int error_sleep = 0;
	char buffer[CHUNKSIZE];
	int in, out;

	if ((in = (*open_fn)(url)) < 0) {
		fprintf(stderr,"Can't open url %s!\n",url);
		exit(EXIT_FAILURE);
	}

	if ((out = open(outfile, O_CREAT|O_TRUNC|O_WRONLY|O_LARGEFILE, 00644)) < 0) {
		perror("open");
		exit(EXIT_FAILURE);
	}

	printf("Recording from %s for %d seconds...\n", url, duration);

	gettimeofday(&start, NULL);
	curr = start;

	do {
		if (error_sleep) {
			sleep(error_sleep);
			printf("Reconnecting... ");
			if ((in = (*open_fn)(url)) < 0) {
				if (error_sleep < 60)
					error_sleep *= 2;

				printf("failed\n");
				continue;
			} else {
				printf("succeeded\n");
				error_sleep = 0;
			}
		}

		if ((recvd = recv(in, buffer, CHUNKSIZE, 0)) < 1) {
			error_sleep = 1;
			continue;
		}
		written = 0;
		do {
			if ((bytes = write(out, buffer, recvd-written)) < 0) {
				perror("write");
				exit(EXIT_FAILURE);
			}
			written += bytes;
		} while(written < recvd);

		if (!(++count % GTOD_INTERVAL))
			gettimeofday(&curr, NULL);
	} while (curr.tv_sec < start.tv_sec+duration);

	close(out);
	close(in);
	shutdown(in, SHUT_RDWR);
}

int main(int argc, char **argv)
{
	int duration;
	char *url;
	char *outfile;
	int(*open_fn)(char *);

	if (argc == 4) {
		url = argv[1];
		duration = atoi(argv[2])*60;
		outfile = argv[3];
	} else {
		fprintf(stderr,"Syntax: %s URL duration_in_minutes outfile\n", argv[0]);
		exit(EXIT_FAILURE);
	}

	if (is_http(url)) {
		open_fn = &open_http;
	} else if (is_mcast(url)) {
		open_fn = &open_mcast;
	} else {
		printf("URL %s not supported!\n", url);
		exit(EXIT_FAILURE);
	}

	record(open_fn, url, outfile, duration);

	return EXIT_SUCCESS;
}
